
window.consumption = function (consumption) {

  var languageId, localeTable;

  var formatNumber = function (number, format) {
    if (typeof number !== "number") return "-";
    var lang = languageId;
    var locale = localeTable[lang ? lang.toLowerCase() : ""];

    if (!locale) locale = "dk";
    if (!format) format = "0.##";
    return $.formatNumber(number, {
      format: format,
      locale: locale
    });
  };

  var graphLabelCallback = function (readingList, width) {
    return function (i) {
      if (i === 0 || i > readingList.length) return "";

      if (width < 700 && readingList.length > 14 && i % 2 === 0) return "";

      return readingList[i - 1];
    };
  };

  consumption.flot = function (args, postDrawCb) {
    languageId = args.languageId;
    localeTable = args.localeTable;

    var $container = args.$container,
        options = args.options,
        yAxisMax = args.yAxisMax,
        yAxisMin = args.yAxisMin;


    $container.unbind("plotclick");

    var width = $container.width();
    var series = args.series;

    var flotoptions = {
      series: {
        shadowSize: 0
      },
      xaxis: {
        tickFormatter: graphLabelCallback(args.xAxisLabels, width),
        tickLength: 0,
        ticks: args.xAxisLabels.length,
        labelHeight: 31,
        min: args.xAxisMin,
        max: args.xAxisMax,
        font: {
          weight: "bold",
          size: 12,
          color: options.colorGridFont
        }
      },
      yaxis: {
        reserveSpace: false,
        tickFormatter: function (val, axis) {
          return formatNumber(parseFloat(val.toFixed(3)), args.numberFormat);
        },
        font: {
          size: 12,
          color: options.colorGridFont
        }
      },
      legend: {
        show: false
      },
      grid: {
        borderWidth: {
          top: 0,
          left: 0,
          bottom: 3,
          right: 0
        },
        borderColor: options.colorGridFont,
        labelMargin: 15,
        backgroundColor: options.colorGridBackground,
        hoverable: true,
        clickable: true,
        autoHighlight: true
      },
      hooks: {
        drawOverlay: [function (plot, eventHolder) {
          if (typeof postDrawCb === "function") {
            postDrawCb();
          }
        }]

      }
    };

    if (yAxisMax && yAxisMax <= 10) {
      flotoptions.yaxis.max = yAxisMax;
      var temp = yAxisMax;
      while (temp <= 1) temp = temp * 10;
      if (temp > 5) {
        temp = temp / 2;
        temp = Math.ceil(temp);
      }
      flotoptions.yaxis.ticks = temp;
    } else if (yAxisMax) {
      flotoptions.yaxis.max = yAxisMax;
    }

    if (yAxisMax === 0) flotoptions.yaxis.max = 0;

    if (yAxisMin === 0) flotoptions.yaxis.min = 0;

    var plot = $.plot($container, series, flotoptions);
    args.previousPoint = null;

    $container.bind("plotclick", function (event, pos, item) {
      Ti.App.fireEvent("plotclick", {
        item: item
      });
    });
  };

  return consumption;
}(window.consumption || {});