

(function ($) {
  function init(initPlot) {
    var nbOfBarsToOrder = 0;
    var pixelInXWidthEquivalent = 1;
    var isHorizontal = false;
    var totalWidth = 0;
    var orderBarWidths = [];

    function serieNeedToBeReordered(serie) {
      return serie.bars != null && serie.bars.show && serie.bars.order != null;
    }

    function detectOrderCount(plot) {
      if (nbOfBarsToOrder > 0) return;
      var series = plot.getData();
      for (var i = 0; i < series.length; i++) {
        if (!series[i].bars.order) series[i].bars.order = 1;
        if (nbOfBarsToOrder < series[i].bars.order) nbOfBarsToOrder = series[i].bars.order;
      }
    }

    function checkIfGraphIsHorizontal(serie) {
      if (serie.bars.horizontal) {
        isHorizontal = true;
      }
    }

    function getAxeMinMaxValues(series, AxeIdx) {
      var minMaxValues = new Array();
      for (var i = 0; i < series.length; i++) {
        minMaxValues[0] = series[i].data[0][AxeIdx];
        minMaxValues[1] = series[i].data[series[i].data.length - 1][AxeIdx];
      }
      if (typeof minMaxValues[0] == 'string') {
        minMaxValues[0] = 0;
        minMaxValues[1] = series[0].data.length - 1;
      }
      return minMaxValues;
    }

    function calculPixel2XWidthConvert(plot) {
      var gridDimSize = isHorizontal ? plot.getPlaceholder().innerHeight() : plot.getPlaceholder().innerWidth();
      var minMaxValues = isHorizontal ? getAxeMinMaxValues(plot.getData(), 1) : getAxeMinMaxValues(plot.getData(), 0);
      var axeSize = minMaxValues[1] - minMaxValues[0];
      pixelInXWidthEquivalent = axeSize / gridDimSize;
    }

    function isCenterOrder(serie) {
      if (nbOfBarsToOrder % 2 === 0) return false;

      var centerOrder = nbOfBarsToOrder / 2 + 0.5;
      return centerOrder === serie.bars.order;
    }

    function sumWidth(plot) {
      if (totalWidth > 0) return totalWidth;

      var seriesGroupedByOrder = _.groupBy(plot.getData(), function (serie) {
        return serie.bars.order;
      });

      _.each(seriesGroupedByOrder, function (seriesGroup) {
        var lineWidth = seriesGroup[0].bars.lineWidth;
        if (!lineWidth) lineWidth = 0;
        var lineWidthInProcentage = lineWidth * 2 * pixelInXWidthEquivalent;
        orderBarWidths[seriesGroup[0].bars.order] = seriesGroup[0].bars.barWidth + lineWidthInProcentage;
        totalWidth += orderBarWidths[seriesGroup[0].bars.order];
      });

      return totalWidth;
    }

    function getCurrentOrderOffset(serie) {
      var resultat = 0;
      var index = 1;
      while (serie.bars.order >= index) {
        if (serie.bars.order === index) resultat += orderBarWidths[index] / 2;else resultat += orderBarWidths[index];
        index += 1;
      }
      var haftWidth = totalWidth / 2;
      return haftWidth - resultat;
    };

    function shiftPoints(datapoints, serie, dx) {
      var ps = datapoints.pointsize;
      var points = datapoints.points;
      var j = 0;
      for (var i = isHorizontal ? 1 : 0; i < points.length; i += ps) {
        points[i] += dx;

        serie.data[j][3] = points[i];
        j++;
      }

      return points;
    }

    function reOrderBars(plot, serie, datapoints) {
      var shiftedPoints = null;

      if (!serieNeedToBeReordered(serie)) return datapoints.points;
      detectOrderCount(plot);
      if (isCenterOrder(serie)) return datapoints.points;

      checkIfGraphIsHorizontal(serie);
      calculPixel2XWidthConvert(plot);
      sumWidth(plot);

      var currentOrderOffset = getCurrentOrderOffset(serie);
      var offsetFactor = currentOrderOffset;
      shiftedPoints = shiftPoints(datapoints, serie, -offsetFactor);
      datapoints.points = shiftedPoints;
      return datapoints.points;
    }

    initPlot.hooks.processDatapoints.push(reOrderBars);
  }

  var options = {
    series: {
      bars: { order: null } }
  };

  $.plot.plugins.push({
    init: init,
    options: options,
    name: "orderBars_custom",
    version: "0.2"
  });
})(jQuery);